#include <Adafruit_GFX.h>
#include <Adafruit_PCD8544.h>

#include "font.h"

#define RST 3
#define CE  4
#define DC  5
#define DIN  8
#define CLK  7
#define SW 2
#define X_pin 0
#define Y_pin 1

Adafruit_PCD8544 display = Adafruit_PCD8544(7, 8, 5, 4, 3);

void LcdXY(int x, int y){
  LcdWriteCmd(0x80 | x); //column
  LcdWriteCmd(0x40 | y); //column
}

void LcdWriteString(char *characters) {
  while(*characters)
    LcdWriteCharacter(*characters++);
}

void LcdWriteCharacter(char character)
{
  for(int i = 0; i < 5; i++)
    LcdWriteData(ASCII[character - 0x20][i]);
  LcdWriteData(0x00);
}

void LcdWriteData(byte dat)
{
  digitalWrite(DC, HIGH); //DC pin is high for commands
  digitalWrite(CE, LOW);
  shiftOut(DIN, CLK, MSBFIRST, dat); //transmit serial data
  digitalWrite(CE, HIGH);
}
void LcdWriteCmd(byte cmd)
{
  digitalWrite(DC, LOW); //DC pin is low for commands
  digitalWrite(CE, LOW);
  shiftOut(DIN, CLK, MSBFIRST, cmd); //transmit serial data
  digitalWrite(CE, HIGH);
}
void clearScreen() {
  for(int i = 0; i < 504; i++) {
    LcdWriteData(0x00);
  }
}

ISR(TIMER1_COMPA_vect) {
  // blink led
  PORTD ^= (1 << PD6);
}

void setup()
{
  DDRD |= (1 << PD6);
  pinMode(13, OUTPUT);
  // inițializare timer 1
  cli();
  TCCR1A = 0;
  TCCR1B = 0;
  TCNT1  = 0;
  OCR1A = 34;            // compare match register 16MHz/256/2Hz-1
  TCCR1B |= (1 << WGM12);   // CTC mode
  TCCR1B |= (1 << CS11) | (1 << CS10);
  TIMSK1 |= (1 << OCIE1A);  // enable timer compare interrupt
  sei();
  
  pinMode(RST, OUTPUT);
  pinMode(CE, OUTPUT);
  pinMode(DC, OUTPUT);
  pinMode(DIN, OUTPUT);
  pinMode(CLK, OUTPUT);
  pinMode(SW, INPUT);
  digitalWrite(SW, HIGH);
  digitalWrite(RST, LOW);
  digitalWrite(RST, HIGH);
  Serial.begin(57600);
  
  LcdWriteCmd(0x21);  // LCD extended commands
  LcdWriteCmd(0xBF);  // set LCD Vop (contrast)
  LcdWriteCmd(0x04);  // set temp coefficent
  LcdWriteCmd(0x14);  // LCD bias mode 1:40
  LcdWriteCmd(0x20);  // LCD basic commands
  LcdWriteCmd(0x0C);  // LCD all segments on
  
  display.begin();
  display.clearDisplay();         
  LcdXY(15, 1);
  LcdXY(15, 1);
  LcdWriteString("Sokoban 10");
  delay(2000);
  LcdXY(25, 3);
  LcdWriteString("Start");
 
}

int Level(int lvl) {
  int xp = 1, yp = 1; //pion
  int nr_blocks = lvl;
  int win = 0;
  int x, y;
  int startx, starty; 
  char *mapa;
  int time = 60;
  if (lvl == 1) {
    mapa = (char*)calloc(15, sizeof(char));
    for (int i = 0; i < 15; i++)
      mapa[i] = unu[i];
    startx = 10;
    starty = 1;
    x = 3;
    y = 5;
  } else if (lvl == 2) {
    mapa = (char*)calloc(48, sizeof(char));
    for (int i = 0; i < 48; i++)
      mapa[i] = doi[i];
    startx = 0;
    starty = 0;
    x = 6;
    y = 8;
  } else if (lvl == 3) {
    mapa = (char*)calloc(54, sizeof(char));
    for (int i = 0; i < 54; i++)
      mapa[i] = trei[i];
    startx = 0;
    starty = 0;
    x = 6;
    y = 9;
  }
  char string[3];
  while (!win) {
    if (time == 0)
      break;
    //afisare tabla lcd joc
    for (int i = 0; i < x; i++) {
      LcdXY(startx, starty + i);
      for (int j = 0; j < y; j++) {
        if (mapa[i * y + j] == '0') {
          LcdWriteCharacter('+');  
        } else {
          LcdWriteCharacter(mapa[i * y + j]);
        }
      }
    }
    LcdXY(60, 1);
    LcdWriteString("Time");
    LcdXY(60, 2);
    LcdWriteString("left");
    LcdXY(60, 3);
    LcdWriteString(dtostrf(time, 2, 0, string));
    //verificare end game
    if (nr_blocks == 0)
      win = 1;
      
    //miscare simpla a pionului pe axele x si y
    if (analogRead(Y_pin) >= 800 && mapa[xp * y + yp + 1] == ' ') {
      mapa[xp * y + yp + 1] = '@';
      mapa[xp * y + yp] = ' ';
      yp += 1;
    }
    if (analogRead(Y_pin) < 300 && mapa[xp * y + yp - 1] == ' ') {
      mapa[xp * y + yp - 1] = '@';
      mapa[xp * y + yp] = ' ';
      yp -= 1;
    }
    if (analogRead(X_pin) < 300 && mapa[(xp + 1) * y + yp] == ' ') {
      mapa[(xp + 1) * y + yp] = '@';
      mapa[xp * y + yp] = ' ';
      xp += 1;
    }
    if (analogRead(X_pin) >= 800 && mapa[(xp - 1) * y + yp] == ' ') {
      mapa[(xp - 1) * y + yp] = '@';
      mapa[xp * y + yp] = ' ';
      xp -= 1;
    }
    
    //miscari cu blocuri
    if (analogRead(Y_pin) >= 800 && mapa[xp * y + yp + 1] == '+' &&
        (mapa[xp * y + yp + 2] == ' ' || mapa[xp * y + yp + 2] == '!')) {
      //daca este spatiu este ignorat, daca este ! este blocul a ajuns pe pozitie
      if (mapa[xp * y + yp + 2] == '!') {
        nr_blocks -= 1;
        mapa[xp * y + yp + 2] = '0';
      } else {
        mapa[xp * y + yp + 2] = '+';        
      }
      mapa[xp * y + yp] = ' ';
      mapa[xp * y + yp + 1] = '@';
      yp += 1;
    }
    if (analogRead(Y_pin) < 300 && mapa[xp * y + yp - 1] == '+' &&
        (mapa[xp * y + yp - 2] == ' ' || mapa[xp * y + yp - 2] == '!')) {
      //daca este spatiu este ignorat, daca este ! este blocul a ajuns pe pozitie
      if (mapa[xp * y + yp - 2] == '!') {
        nr_blocks -= 1;
        mapa[xp * y + yp - 2] = '0';
      } else {
        mapa[xp * y + yp - 2] = '+';        
      }
      mapa[xp * y + yp] = ' ';
      mapa[xp * y + yp - 1] = '@';

      yp -= 1;
    }
    if (analogRead(X_pin) >= 800 && mapa[(xp - 1) * y + yp] == '+' &&
        (mapa[(xp - 2) * y + yp] == ' ' || mapa[(xp - 2) * y + yp] == '!')) {
      //daca este spatiu este ignorat, daca este ! este blocul a ajuns pe pozitie
      if (mapa[(xp - 2) * y + yp] == '!') {
        nr_blocks -= 1;
        mapa[(xp - 2) * y + yp] = '0';
      } else {
        mapa[(xp - 2) * y + yp] = '+';        
      }
      mapa[xp * y + yp] = ' ';
      mapa[(xp - 1) * y + yp] = '@';
      xp -= 1;
    }
    if (analogRead(X_pin) < 300 && mapa[(xp + 1) * y + yp] == '+' &&
        (mapa[(xp + 2) * y + yp] == ' ' || mapa[(xp + 2) * y + yp] == '!')) {
      //daca este spatiu este ignorat, daca este ! este blocul a ajuns pe pozitie
      if (mapa[(xp + 2) * y + yp] == '!') {
        nr_blocks -= 1;
        mapa[(xp + 2) * y + yp] = '0';
      } else {
        mapa[(xp + 2) * y + yp] = '+';        
      }
      mapa[xp * y + yp] = ' ';
      mapa[(xp + 1) * y + yp] = '@';
      xp += 1;
    }
    digitalWrite(13, HIGH);
    OCR1A = 35;
    delay(150);
    digitalWrite(13, LOW);
    OCR1A = 280;
    delay(100);
    time -= 1;
  }
  free(mapa);
  if (time == 0)
    return -1;
  return 0;
  
}

void loop()
{
  Serial.print(digitalRead(SW));
  if(digitalRead(SW) == 0) {
    clearScreen();
    LcdXY(20, 2);
    LcdWriteString("Level 1");
    delay(1000);
    clearScreen();
    if (Level(1) == 0) {
      clearScreen();
      LcdXY(0, 2);
      LcdWriteString("Level complete");
      delay(1000);
      clearScreen();
      LcdXY(20, 2);
      LcdWriteString("Level 2");
      delay(1000);
      clearScreen();
      if (Level(2) == 0) {
        clearScreen();
        LcdXY(0, 2);
        LcdWriteString("Level complete");
        delay(1000);
        clearScreen();
        LcdXY(20, 2);
        LcdWriteString("Level 3");
        delay(1000);
        clearScreen();
        if (Level(3) == 0) {
          clearScreen();
          LcdXY(10, 2);
          LcdWriteString("You won!:)");
        }else {
          clearScreen();
          LcdXY(10, 2);
          LcdWriteString("You lose:(");
        }
     }else {
      clearScreen();
      LcdXY(10, 2);
      LcdWriteString("You lose:(");
     }
    } else {
      clearScreen();
      LcdXY(10, 2);
      LcdWriteString("You lose:(");
    }
  }
}
